﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using FluorineFx;
using FluorineFx.Messaging;
using FluorineFx.Messaging.Api;
using FluorineFx.Messaging.Api.SO;
using FluorineFx.Net;

namespace FluorineFxSharedObjectTest
{
    class Program
    {
        private NetConnection _netConnection;
        private RemoteSharedObject _sharedObject;

        static void Main(string[] args)
        {
            Program p = new Program();
            p.Connect();

            Console.ReadLine();
            p.Read();
            Console.ReadLine();
            p.Update1();
            Console.ReadLine();
            p.Update2();
            Console.ReadLine();
            p.Read();
            Console.ReadLine();
        }

        private void Connect()
        {
            _netConnection = new NetConnection();
            _netConnection.OnConnect += new ConnectHandler(netConnection_OnConnect);
            _netConnection.NetStatus += new NetStatusHandler(netConnection_NetStatus);
            _netConnection.OnDisconnect += new DisconnectHandler(netConnection_OnDisconnect);
            _netConnection.Client = this;
            _netConnection.Connect("rtmp://localhost:1935/sotest");
        }

        public void Update1()
        {
            try
            {
                _sharedObject.BeginUpdate();
                _sharedObject.SetAttribute("Id", 1);
                _sharedObject.SetAttribute("Name", "Jiang.Yi.1");
                _sharedObject.SetAttribute("Date", DateTime.Now);
                _sharedObject.SetAttribute("IsOk", true);
                var child = new Child() { Id = 1, Name = "jojo.1" };
                _sharedObject.SetAttribute("Child", child);
            }
            finally
            {
                _sharedObject.EndUpdate();
            }
        }

        public void Update2()
        {
            try
            {
                _sharedObject.BeginUpdate();
                _sharedObject.SetAttribute("Id", 2);
                _sharedObject.SetAttribute("Name", "Jiang.Yi.2");
                _sharedObject.SetAttribute("Date", DateTime.Now.AddYears(1));
                _sharedObject.SetAttribute("IsOk", false);
                var child = new Child() { Id = 2, Name = "jojo.2" };
                _sharedObject.SetAttribute("Child", child);
            }
            finally
            {
                _sharedObject.EndUpdate();
            }
        }

        public void Read()
        {
            var id = _sharedObject.GetAttribute("Id");
            var name = _sharedObject.GetAttribute("Name");
            var date = _sharedObject.GetAttribute("Date");
            var isOk = _sharedObject.GetAttribute("IsOk");
            var child = _sharedObject.GetAttribute("Child") as Child;
            Console.WriteLine(
                "Read ---> id:" + id
                + ",name:" + name
                + ",isOk:" + isOk
                + ",date:" + date
                + ",child:" + (child != null ? ("{" + child.Id + "," + child.Name + "}") : "null")
                );
        }

        public void Delete()
        {
            _sharedObject.RemoveAttributes();
        }

        void netConnection_OnConnect(object sender, EventArgs e)
        {
            Console.WriteLine("netConnection_OnConnect" + Environment.NewLine);
            _sharedObject = RemoteSharedObject.GetRemote("TestRSO", _netConnection.Uri.ToString(), true);
            _sharedObject.OnConnect += new ConnectHandler(sharedObject_OnConnect);
            _sharedObject.OnDisconnect += new DisconnectHandler(sharedObject_OnDisconnect);
            _sharedObject.NetStatus += new NetStatusHandler(sharedObject_NetStatus);
            _sharedObject.Sync += new SyncHandler(sharedObject_Sync);
            _sharedObject.Connect(_netConnection);
        }
        void netConnection_NetStatus(object sender, NetStatusEventArgs e)
        {
            Console.WriteLine("netConnection_NetStatus: level: " + e.Info["level"] + ",code: " + e.Info["code"] + Environment.NewLine);
        }
        void netConnection_OnDisconnect(object sender, EventArgs e)
        {
            _sharedObject.Close();
            _netConnection.close();
        }

        private void sharedObject_Sync(object sender, SyncEventArgs e)
        {
            string s = string.Empty;
            try
            {
                foreach (var asObject in e.ChangeList)
                {
                    var code = asObject["code"];
                    if (code == "change")
                    {
                        s += asObject["code"] + ":" + asObject["name"] + ", oldValue:" + asObject["oldValue"] + ";\r\n";
                    }
                    else
                    {
                        s += asObject["code"] + ":" + asObject["name"] + ";\r\n";
                    }
                }
            }
            catch
            {
            }

            Console.WriteLine("sharedObject_Sync:\r\n" + s);
        }
        private void sharedObject_NetStatus(object sender, NetStatusEventArgs e)
        {
            Console.WriteLine("sharedObject_NetStatus: " + e.Info["level"] + ", code:" + e.Info["code"] + Environment.NewLine);
        }
        private void sharedObject_OnDisconnect(object sender, EventArgs e)
        {
            Console.WriteLine("sharedObject_OnDisconnect" + Environment.NewLine);
        }
        private void sharedObject_OnConnect(object sender, EventArgs e)
        {
            Console.WriteLine("sharedObject_OnConnect" + Environment.NewLine);
            //Delete();
        }
    }
}
